/*
 * Copyright (c) 2014 Samsung Electronics Co., Ltd All Rights Reserved
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <unistd.h>
#include <app.h>
#include <app_manager.h>
#include <vconf.h>
#include <vconf-internal-location-keys.h>

#include <common/ApplicationContext.h>

#include "heremaps-uc-dbus.h"
#include "here_userconsent.h"

HERE_PLUGIN_BEGIN_NAMESPACE

static void UserConsentChangedCb(keynode_t *node EINA_UNUSED, void *user_data)
{
	MAPS_FUNC_ENTER

	HereUserConsent *pUserConsent = (HereUserConsent *)user_data;

	if (!pUserConsent->m_pCbFunc) {
		MAPS_LOGD("m_pCbFunc and  is NULL");
		delete pUserConsent;
		return;
	}

	int enabled = 0;
	vconf_get_int(VCONFKEY_LOCATION_HEREMAPS_CONSENT, &enabled);
	if (enabled != 0 && enabled != 1)
		return;
	vconf_ignore_key_changed(VCONFKEY_LOCATION_HEREMAPS_CONSENT, UserConsentChangedCb);
	pUserConsent->isSetVconfCb = false;

	MAPS_LOGD("Vconf value of HereMaps is %d", enabled);
	ApplicationContext::GetInstance().SetUserConsent(enabled);

	if (pUserConsent->m_provider)
		((maps_service_request_user_consent_cb)pUserConsent->m_pCbFunc)((bool)enabled, pUserConsent->m_provider, pUserConsent->m_pUserData);
	else
		((maps_service_request_user_consent_with_handle_cb)pUserConsent->m_pCbFunc)((bool)enabled, pUserConsent->m_pUserData);

	delete pUserConsent;

	MAPS_FUNC_EXIT
}

HereUserConsent::HereUserConsent(const char *provider, void *pCbFunc, void *pUserData)
{
	isSetVconfCb = false;
	m_provider = provider;
	m_pCbFunc = pCbFunc;
	m_pUserData = pUserData;
}

HereUserConsent::~HereUserConsent()
{
	if (isSetVconfCb)
		vconf_ignore_key_changed(VCONFKEY_LOCATION_HEREMAPS_CONSENT, UserConsentChangedCb);
	isSetVconfCb = false;
}

here_error_e HereUserConsent::GetUserConsent(bool *consent)
{
	if (ApplicationContext::GetInstance().GetUserConsent() == true) {
		*consent = true;
		return HERE_ERROR_NONE;
	}

	const int testAppIdCount = 8;
	const char* testAppIdList[testAppIdCount] = {
		"org.tizen.capi-maps-service-native-utc",
		"org.tizen.capi-maps-service-native-itc",
		"core.capi-maps-service-tests",
		"native.capi-maps-service-itc",
		"Tizen.Maps.Test",
		"Tizen.Maps.Manual.Test",
		"Tizen.XamarinFormsMaps.Tests",
		"Tizen.XamarinFormsMaps.Manual.Tests",
	};

	int enabled = 0;
	int ret = 0;

	ret = vconf_get_int(VCONFKEY_LOCATION_HEREMAPS_CONSENT, &enabled);
	MAPS_LOGD("VCONFKEY_LOCATION_HEREMAPS_CONSENT is %d", enabled);
	if (ret != 0) {
		MAPS_LOGD("Fail to get vconf value");
		*consent = FALSE;
		return HERE_ERROR_SERVICE_NOT_AVAILABLE;
	}
	if (enabled != 1) {
		pid_t nProcessId = getpid();
		char *strAppId = NULL;

		ret = app_manager_get_app_id(nProcessId, &strAppId);
		if (ret != APP_MANAGER_ERROR_NONE) {
			MAPS_LOGI("Get app_id [%d]. nRet[%d]", (int)nProcessId, ret);
		} else {
			for (int i = 0; i < testAppIdCount; i++) {
				if (!strncmp(strAppId, testAppIdList[i], strlen(testAppIdList[i]))) {
					MAPS_LOGD("Requested by tct");
					enabled = 1;
					break;
				}
			}
		}
		g_free(strAppId);
	}
	*consent = (enabled == 1);
	ApplicationContext::GetInstance().SetUserConsent(*consent);

	return HERE_ERROR_NONE;
}

here_error_e HereUserConsent::SetUserConsent(bool consent)
{
	int enabled = (int)consent;
	int ret = 0;

	ret = vconf_set_int(VCONFKEY_LOCATION_HEREMAPS_CONSENT, enabled);
	MAPS_LOGD("set VCONFKEY_LOCATION_HEREMAPS_CONSENT : %d", enabled);
	if (ret != 0) {
		MAPS_LOGD("Fail to set vconf value");
		return HERE_ERROR_SERVICE_NOT_AVAILABLE;
	}
	ApplicationContext::GetInstance().SetUserConsent(enabled);
	return HERE_ERROR_NONE;
}

here_error_e HereUserConsent::RequestUserConsent()
{
	bool agreed = false;
	HereUserConsent::GetUserConsent(&agreed);

	if (agreed != TRUE) {
		if (vconf_notify_key_changed(VCONFKEY_LOCATION_HEREMAPS_CONSENT, UserConsentChangedCb, this) < 0) {
			MAPS_LOGD("Unable to register a vconf changed cb to VCONFKEY_LOCATION_HEREMAPS_CONSENT");
			delete this;
			return HERE_ERROR_SERVICE_NOT_AVAILABLE;
		}
		isSetVconfCb = true;
		int ret = HEREMAPS_UC_DBUS_ERROR_NONE;
		MAPS_LOGD("heremaps_uc_dbus_launch_receiver is called");
		ret = heremaps_uc_dbus_launch_receiver();
		if (ret != HEREMAPS_UC_DBUS_ERROR_NONE)
			MAPS_LOGD("heremaps_uc_dbus_launch_receiver fail");
	} else {
		MAPS_LOGD("Vconf value of HereMaps is true");
		if (m_provider)
			((maps_service_request_user_consent_cb)m_pCbFunc)(TRUE, m_provider, m_pUserData);
		else
			((maps_service_request_user_consent_with_handle_cb)m_pCbFunc)(TRUE, m_pUserData);

		delete this;
	}

	return HERE_ERROR_NONE;
}

HERE_PLUGIN_END_NAMESPACE
